@@ -1,17 +1,18 @@
 import logging
 from abc import ABC, abstractmethod
+from enum import Enum
 from itertools import count
-from typing import Final, Iterable, Optional, Tuple, Union
+from typing import Any, Dict, Final, Iterable, Optional, Tuple, Union
 
 import attr
 
 from .api_types import (APIResponse, BaseTelegram, BotCommand, Chat,
-                        ChatMember, ChatPermissions, File, GameHighScore,
-                        InlineKeyboardMarkup, InlineQueryResult, InputFile,
-                        InputMedia, InputMediaAudio, InputMediaDocument,
-                        InputMediaPhoto, InputMediaVideo, LabeledPrice,
-                        MaskPosition, Message, MessageEntity, MessageId,
-                        PassportElementError, Poll, ReplyMarkup,
+                        ChatInviteLink, ChatMember, ChatPermissions, File,
+                        GameHighScore, InlineKeyboardMarkup, InlineQueryResult,
+                        InputFile, InputMedia, InputMediaAudio,
+                        InputMediaDocument, InputMediaPhoto, InputMediaVideo,
+                        LabeledPrice, MaskPosition, Message, MessageEntity,
+                        MessageId, PassportElementError, Poll, ReplyMarkup,
                         ShippingOption, StickerSet, Update, User,
                         UserProfilePhotos, WebhookInfo)
 from .constants import (ChatAction, DiceEmoji, ParseMode, PollType,
@@ -23,27 +24,40 @@ __all__ = ('ParamType', 'ApiMethods')
 api_logger: Final[logging.Logger] = logging.getLogger('aiotgbot.api')
 
 
+_ItemType = Union[BaseTelegram, str, int, Enum]
+
+
+def _unstructure_item(
+    item: Union[BaseTelegram, str, int, Enum]
+) -> Union[Dict[str, Any], str, int]:
+    if isinstance(item, BaseTelegram):
+        return item.to_dict()
+    elif isinstance(item, Enum) and isinstance(item.value, str):
+        return item.value
+    elif isinstance(item, Enum):
+        raise TypeError(f'Unsupported enum type: {item!r}')
+    elif isinstance(item, (str, int)):
+        return item
+    else:
+        raise TypeError(f'Unsupported item type: {item!r}')
+
+
 def _json_dumps(
-    value: Union[BaseTelegram, Iterable[BaseTelegram], None]
+    value: Union[BaseTelegram, Iterable[_ItemType], None]
 ) -> Optional[str]:
     if value is None:
         return None
     elif isinstance(value, Iterable):
-        return json_dumps(tuple(item.to_dict() for item in value))
-    else:
+        return json_dumps(tuple(_unstructure_item(item) for item in value))
+    elif isinstance(value, BaseTelegram):
         return json_dumps(value.to_dict())
+    else:
+        raise TypeError(f'Unsupported value type: {value!r}')
 
 
-def _strs_to_json(value: Optional[Iterable[str]]) -> Optional[str]:
-    return json_dumps(tuple(value)) if value is not None else None
-
-
-def _parse_mode_to_str(parse_mode: Optional[ParseMode]) -> Optional[str]:
-    return parse_mode.value if parse_mode is not None else None
-
-
-def _dice_emoji_to_str(dice_emoji: Optional[DiceEmoji]) -> Optional[str]:
-    return dice_emoji.value if dice_emoji is not None else None
+def _enum_to_str(item: Optional[Enum]) -> Optional[str]:
+    assert item is None or isinstance(item.value, str)
+    return item.value if item is not None else None
 
 
 ParamType = Union[int, float, str, InputFile, None]
@@ -72,7 +86,7 @@ class ApiMethods(ABC):
                          f'allowed_updates: {allowed_updates}')
         response = await self._request(
             RequestMethod.GET, 'getUpdates', offset=offset, limit=limit,
-            timeout=timeout, allowed_updates=_strs_to_json(allowed_updates))
+            timeout=timeout, allowed_updates=_json_dumps(allowed_updates))
 
         return tuple(Update.from_dict(item) for item in response.result)
 
@@ -88,7 +102,7 @@ class ApiMethods(ABC):
         response = await self._request(
             RequestMethod.POST, 'setWebhook', url=url, certificate=certificate,
             ip_address=ip_address, max_connections=max_connections,
-            allowed_updates=_strs_to_json(allowed_updates),
+            allowed_updates=_json_dumps(allowed_updates),
             drop_pending_updates=drop_pending_updates)
         assert isinstance(response.result, bool)
         return response.result
@@ -139,7 +153,7 @@ class ApiMethods(ABC):
         api_logger.debug('Send message %r to chat "%s"', text, chat_id)
         response = await self._safe_request(
             RequestMethod.POST, 'sendMessage', chat_id, text=text,
-            parse_mode=_parse_mode_to_str(parse_mode),
+            parse_mode=_enum_to_str(parse_mode),
             entities=_json_dumps(entities),
             disable_web_page_preview=disable_web_page_preview,
             disable_notification=disable_notification,
@@ -200,7 +214,7 @@ class ApiMethods(ABC):
         api_logger.debug('Send photo to "%s"', chat_id)
         response = await self._safe_request(
             RequestMethod.POST, 'sendPhoto', chat_id, photo=photo,
-            caption=caption, parse_mode=_parse_mode_to_str(parse_mode),
+            caption=caption, parse_mode=_enum_to_str(parse_mode),
             caption_entities=_json_dumps(caption_entities),
             disable_notification=disable_notification,
             reply_to_message_id=reply_to_message_id,
@@ -227,7 +241,7 @@ class ApiMethods(ABC):
         api_logger.debug('Send audio to "%s"', chat_id)
         response = await self._safe_request(
             RequestMethod.POST, 'sendAudio', chat_id, audio=audio,
-            caption=caption, parse_mode=_parse_mode_to_str(parse_mode),
+            caption=caption, parse_mode=_enum_to_str(parse_mode),
             caption_entities=_json_dumps(caption_entities),
             disable_notification=disable_notification,
             duration=duration, performer=performer,
@@ -258,7 +272,7 @@ class ApiMethods(ABC):
         response = await self._safe_request(
             RequestMethod.POST, 'sendDocument', chat_id,
             document=document, thumb=thumb, caption=caption,
-            parse_mode=_parse_mode_to_str(parse_mode),
+            parse_mode=_enum_to_str(parse_mode),
             caption_entities=_json_dumps(caption_entities),
             disable_content_type_detection=disable_content_type_detection,
             disable_notification=disable_notification,
@@ -290,7 +304,7 @@ class ApiMethods(ABC):
             RequestMethod.POST, 'sendVideo', chat_id,
             video=video, duration=duration, width=width,
             height=height, thumb=thumb, caption=caption,
-            parse_mode=_parse_mode_to_str(parse_mode),
+            parse_mode=_enum_to_str(parse_mode),
             caption_entities=_json_dumps(caption_entities),
             supports_streaming=supports_streaming,
             disable_notification=disable_notification,
@@ -320,7 +334,7 @@ class ApiMethods(ABC):
             RequestMethod.POST, 'sendAnimation', chat_id,
             animation=animation, duration=duration, width=width,
             height=height, thumb=thumb, caption=caption,
-            parse_mode=_parse_mode_to_str(parse_mode),
+            parse_mode=_enum_to_str(parse_mode),
             caption_entities=_json_dumps(caption_entities),
             disable_notification=disable_notification,
             reply_to_message_id=reply_to_message_id,
@@ -345,7 +359,7 @@ class ApiMethods(ABC):
         response = await self._safe_request(
             RequestMethod.POST, 'sendVoice', chat_id,
             voice=voice, caption=caption,
-            parse_mode=_parse_mode_to_str(parse_mode),
+            parse_mode=_enum_to_str(parse_mode),
             caption_entities=_json_dumps(caption_entities),
             duration=duration,
             disable_notification=disable_notification,
@@ -400,7 +414,7 @@ class ApiMethods(ABC):
                     item, media=f'attach://{attachment_name}'))
         response = await self._safe_request(
             RequestMethod.POST, 'sendMediaGroup', chat_id,
-            media=json_dumps(tuple(item.to_dict() for item in attached_media)),
+            media=_json_dumps(attached_media),
             disable_notification=disable_notification,
             reply_to_message_id=reply_to_message_id,
             allow_sending_without_reply=allow_sending_without_reply,
@@ -533,10 +547,10 @@ class ApiMethods(ABC):
 
     async def send_poll(
         self, chat_id: Union[int, str], question: str,
-        options: Iterable[str], is_anonymous: Optional[bool],
+        options: Iterable[str], is_anonymous: Optional[bool] = None,
         type_: Optional[PollType] = None,
         allows_multiple_answers: Optional[bool] = None,
-        correct_option_id: Optional[bool] = None,
+        correct_option_id: Optional[int] = None,
         is_closed: Optional[bool] = None,
         explanation: Optional[str] = None,
         explanation_parse_mode: Optional[ParseMode] = None,
@@ -551,7 +565,7 @@ class ApiMethods(ABC):
         api_logger.debug('Send poll to chat "%s"', chat_id)
         response = await self._safe_request(
             RequestMethod.POST, 'sendPoll', chat_id,
-            question=question, options=json_dumps(tuple(options)),
+            question=question, options=_json_dumps(options),
             is_anonymous=is_anonymous, type=type_,
             allows_multiple_answers=allows_multiple_answers,
             correct_option_id=correct_option_id,
@@ -575,7 +589,7 @@ class ApiMethods(ABC):
         api_logger.debug('Send dice "%s" to chat "%s"', emoji, chat_id)
         response = await self._safe_request(
             RequestMethod.POST, 'sendDice', chat_id,
-            emoji=_dice_emoji_to_str(emoji),
+            emoji=_enum_to_str(emoji),
             disable_notification=disable_notification,
             reply_to_message_id=reply_to_message_id,
             allow_sending_without_reply=allow_sending_without_reply,
@@ -611,10 +625,12 @@ class ApiMethods(ABC):
         return File.from_dict(response.result)
 
     async def kick_chat_member(self, chat_id: Union[int, str], user_id: int,
-                               until_date: Optional[int] = None) -> bool:
+                               until_date: Optional[int] = None,
+                               revoke_messages: Optional[bool] = None) -> bool:
         response = await self._request(
             RequestMethod.POST, 'kickChatMember',
-            chat_id=chat_id, user_id=user_id, until_date=until_date)
+            chat_id=chat_id, user_id=user_id, until_date=until_date,
+            revoke_messages=revoke_messages)
         assert isinstance(response.result, bool)
         return response.result
 
@@ -636,7 +652,7 @@ class ApiMethods(ABC):
         api_logger.debug('Restrict member %s in "%s"', user_id, chat_id)
         response = await self._request(
             RequestMethod.POST, 'restrictChatMember', chat_id=chat_id,
-            user_id=user_id, permissions=json_dumps(permissions.to_dict()),
+            user_id=user_id, permissions=_json_dumps(permissions),
             until_date=until_date)
         assert isinstance(response.result, bool)
         return response.result
@@ -645,10 +661,12 @@ class ApiMethods(ABC):
         self, chat_id: Union[int, str],
         user_id: int,
         is_anonymous: Optional[int] = None,
+        can_manage_chat: Optional[int] = None,
         can_change_info: Optional[int] = None,
         can_post_messages: Optional[bool] = None,
         can_edit_messages: Optional[bool] = None,
         can_delete_messages: Optional[bool] = None,
+        can_manage_voice_chats: Optional[bool] = None,
         can_invite_users: Optional[bool] = None,
         can_restrict_members: Optional[bool] = None,
         can_pin_messages: Optional[bool] = None,
@@ -658,10 +676,12 @@ class ApiMethods(ABC):
         response = await self._request(
             RequestMethod.POST, 'promoteChatMember',
             chat_id=chat_id, user_id=user_id, is_anonymous=is_anonymous,
+            can_manage_chat=can_manage_chat,
             can_change_info=can_change_info,
             can_post_messages=can_post_messages,
             can_edit_messages=can_edit_messages,
             can_delete_messages=can_delete_messages,
+            can_manage_voice_chats=can_manage_voice_chats,
             can_invite_users=can_invite_users,
             can_restrict_members=can_restrict_members,
             can_pin_messages=can_pin_messages,
@@ -689,6 +709,37 @@ class ApiMethods(ABC):
         assert isinstance(response.result, str)
         return response.result
 
+    async def create_chat_invite_link(
+        self, chat_id: Union[int, str], expire_date: Optional[int] = None,
+        member_limit: Optional[int] = None
+    ) -> ChatInviteLink:
+        api_logger.debug('Create chat "%s" invite link', chat_id)
+        response = await self._request(
+            RequestMethod.POST, 'createChatInviteLink', chat_id=chat_id,
+            expire_date=expire_date, member_limit=member_limit)
+        return ChatInviteLink.from_dict(response.result)
+
+    async def edit_chat_invite_link(
+        self, chat_id: Union[int, str], invite_link: str,
+        expire_date: Optional[int] = None,
+        member_limit: Optional[int] = None
+    ) -> ChatInviteLink:
+        api_logger.debug('Edit chat "%s" invite link', chat_id)
+        response = await self._request(
+            RequestMethod.POST, 'editChatInviteLink', chat_id=chat_id,
+            invite_link=invite_link, expire_date=expire_date,
+            member_limit=member_limit)
+        return ChatInviteLink.from_dict(response.result)
+
+    async def revoke_chat_invite_link(
+        self, chat_id: Union[int, str], invite_link: str
+    ) -> ChatInviteLink:
+        api_logger.debug('Revoke chat "%s" invite link', chat_id)
+        response = await self._request(
+            RequestMethod.POST, 'revokeChatInviteLink', chat_id=chat_id,
+            invite_link=invite_link)
+        return ChatInviteLink.from_dict(response.result)
+
     async def set_chat_permissions(
         self, chat_id: Union[int, str],
         permissions: ChatPermissions
@@ -696,7 +747,7 @@ class ApiMethods(ABC):
         api_logger.debug('Set chat "%s" permissions', chat_id)
         response = await self._safe_request(
             RequestMethod.POST, 'setChatPermissions', chat_id,
-            permissions=json_dumps(permissions.to_dict()))
+            permissions=_json_dumps(permissions))
         assert isinstance(response.result, bool)
         return response.result
 
@@ -833,8 +884,7 @@ class ApiMethods(ABC):
         api_logger.debug('Set my commands "%s"', commands)
         response = await self._request(
             RequestMethod.POST, 'setMyCommands',
-            commands=json_dumps(tuple(command.to_dict()
-                                      for command in commands)))
+            commands=_json_dumps(commands))
         assert isinstance(response.result, bool)
         return response.result
 
@@ -871,7 +921,7 @@ class ApiMethods(ABC):
             RequestMethod.POST, 'editMessageText',
             chat_id=chat_id, message_id=message_id,
             inline_message_id=inline_message_id, text=text,
-            parse_mode=_parse_mode_to_str(parse_mode),
+            parse_mode=_enum_to_str(parse_mode),
             entities=_json_dumps(entities),
             disable_web_page_preview=disable_web_page_preview,
             reply_markup=_json_dumps(reply_markup))
@@ -907,7 +957,7 @@ class ApiMethods(ABC):
             RequestMethod.POST, 'editMessageCaption',
             chat_id=chat_id, message_id=message_id,
             inline_message_id=inline_message_id,
-            caption=caption, parse_mode=_parse_mode_to_str(parse_mode),
+            caption=caption, parse_mode=_enum_to_str(parse_mode),
             caption_entities=_json_dumps(caption_entities),
             reply_markup=_json_dumps(reply_markup))
 
@@ -1103,7 +1153,7 @@ class ApiMethods(ABC):
         response = await self._request(
             RequestMethod.POST, 'answerInlineQuery',
             inline_query_id=inline_query_id,
-            results=json_dumps(tuple(result.to_dict() for result in results)),
+            results=_json_dumps(results),
             cache_time=cache_time, is_personal=is_personal,
             next_offset=next_offset, switch_pm_text=switch_pm_text,
             switch_pm_parameter=switch_pm_parameter)
@@ -1112,8 +1162,11 @@ class ApiMethods(ABC):
 
     async def send_invoice(
         self, chat_id: int, title: str, description: str, payload: str,
-        provider_token: str, start_parameter: str, currency: str,
-        prices: Iterable[LabeledPrice], provider_data: Optional[str] = None,
+        provider_token: str, currency: str, prices: Iterable[LabeledPrice],
+        max_tip_amount: Optional[int] = None,
+        suggested_tip_amounts: Optional[Tuple[int, ...]] = None,
+        start_parameter: Optional[str] = None,
+        provider_data: Optional[str] = None,
         photo_url: Optional[str] = None, photo_size: Optional[int] = None,
         photo_width: Optional[int] = None, photo_height: Optional[int] = None,
         need_name: Optional[bool] = None,
@@ -1132,9 +1185,12 @@ class ApiMethods(ABC):
         response = await self._safe_request(
             RequestMethod.POST, 'sendInvoice', chat_id,
             title=title, description=description, payload=payload,
-            provider_token=provider_token, start_parameter=start_parameter,
+            provider_token=provider_token,
             currency=currency,
-            prices=json_dumps(tuple(price.to_dict() for price in prices)),
+            prices=_json_dumps(prices),
+            max_tip_amount=max_tip_amount,
+            suggested_tip_amounts=_json_dumps(suggested_tip_amounts),
+            start_parameter=start_parameter,
             provider_data=provider_data, photo_url=photo_url,
             photo_size=photo_size, photo_width=photo_width,
             photo_height=photo_height, need_name=need_name,
@@ -1156,15 +1212,10 @@ class ApiMethods(ABC):
         error_message: Optional[str] = None
     ) -> bool:
         api_logger.debug('Answer shipping query "%s"', inline_query_id)
-        if shipping_options is not None:
-            shipping_options_json: Optional[str] = json_dumps(tuple(
-                item.to_dict() for item in shipping_options))
-        else:
-            shipping_options_json = None
         response = await self._request(
             RequestMethod.POST, 'answerShippingQuery',
             inline_query_id=inline_query_id, ok=ok,
-            shipping_options=shipping_options_json,
+            shipping_options=_json_dumps(shipping_options),
             error_message=error_message)
         assert isinstance(response.result, bool)
         return response.result
@@ -1190,7 +1241,7 @@ class ApiMethods(ABC):
         response = await self._request(
             RequestMethod.POST, 'setPassportDataErrors',
             user_id=user_id,
-            errors=json_dumps(tuple(error.to_dict() for error in errors)))
+            errors=_json_dumps(errors))
         assert isinstance(response.result, bool)
         return response.result
 
