@@ -18,32 +18,33 @@ __all__ = ('DataMappingError', 'StreamFile', 'LocalFile', 'BaseTelegram',
            'UserProfilePhotos', 'File', 'ReplyMarkup', 'ReplyKeyboardMarkup',
            'KeyboardButton', 'KeyboardButtonPollType', 'ReplyKeyboardRemove',
            'InlineKeyboardMarkup', 'InlineKeyboardButton', 'LoginUrl',
-           'CallbackQuery', 'ForceReply', 'ChatPhoto', 'ChatMember',
-           'ChatPermissions', 'ChatLocation', 'BotCommand', 'InputFile',
-           'InputMedia', 'InputMediaPhoto', 'InputMediaVideo',
-           'InputMediaAnimation', 'InputMediaAudio', 'InputMediaDocument',
-           'Sticker', 'StickerSet', 'MaskPosition', 'InlineQuery',
-           'InlineQueryResult', 'InlineQueryResultArticle',
-           'InlineQueryResultPhoto', 'InlineQueryResultGif',
-           'InlineQueryResultMpeg4Gif', 'InlineQueryResultVideo',
-           'InlineQueryResultAudio', 'InlineQueryResultVoice',
-           'InlineQueryResultDocument', 'InlineQueryResultLocation',
-           'InlineQueryResultVenue', 'InlineQueryResultContact',
-           'InlineQueryResultGame', 'InlineQueryResultCachedPhoto',
-           'InlineQueryResultCachedGif', 'InlineQueryResultCachedMpeg4Gif',
-           'InlineQueryResultCachedSticker', 'InlineQueryResultCachedDocument',
-           'InlineQueryResultCachedVideo', 'InlineQueryResultCachedVoice',
-           'InlineQueryResultCachedAudio', 'InputMessageContent',
-           'InputTextMessageContent', 'InputLocationMessageContent',
-           'InputVenueMessageContent', 'InputContactMessageContent',
-           'ChosenInlineResult', 'LabeledPrice', 'Invoice', 'ShippingAddress',
-           'OrderInfo', 'ShippingOption', 'SuccessfulPayment', 'ShippingQuery',
-           'PreCheckoutQuery', 'PassportData', 'PassportFile',
-           'EncryptedPassportElement', 'EncryptedCredentials',
-           'PassportElementError', 'PassportElementErrorDataField',
-           'PassportElementErrorFrontSide', 'PassportElementErrorReverseSide',
-           'PassportElementErrorSelfie', 'PassportElementErrorFile',
-           'PassportElementErrorFiles', 'PassportElementErrorTranslationFile',
+           'CallbackQuery', 'ForceReply', 'ChatPhoto', 'ChatInviteLink',
+           'ChatMember', 'ChatMemberUpdated', 'ChatPermissions',
+           'ChatLocation', 'BotCommand', 'InputFile', 'InputMedia',
+           'InputMediaPhoto', 'InputMediaVideo', 'InputMediaAnimation',
+           'InputMediaAudio', 'InputMediaDocument', 'Sticker', 'StickerSet',
+           'MaskPosition', 'InlineQuery', 'InlineQueryResult',
+           'InlineQueryResultArticle', 'InlineQueryResultPhoto',
+           'InlineQueryResultGif', 'InlineQueryResultMpeg4Gif',
+           'InlineQueryResultVideo', 'InlineQueryResultAudio',
+           'InlineQueryResultVoice', 'InlineQueryResultDocument',
+           'InlineQueryResultLocation', 'InlineQueryResultVenue',
+           'InlineQueryResultContact', 'InlineQueryResultGame',
+           'InlineQueryResultCachedPhoto', 'InlineQueryResultCachedGif',
+           'InlineQueryResultCachedMpeg4Gif', 'InlineQueryResultCachedSticker',
+           'InlineQueryResultCachedDocument', 'InlineQueryResultCachedVideo',
+           'InlineQueryResultCachedVoice', 'InlineQueryResultCachedAudio',
+           'InputMessageContent', 'InputTextMessageContent',
+           'InputLocationMessageContent', 'InputVenueMessageContent',
+           'InputContactMessageContent', 'ChosenInlineResult', 'LabeledPrice',
+           'Invoice', 'ShippingAddress', 'OrderInfo', 'ShippingOption',
+           'SuccessfulPayment', 'ShippingQuery', 'PreCheckoutQuery',
+           'PassportData', 'PassportFile', 'EncryptedPassportElement',
+           'EncryptedCredentials', 'PassportElementError',
+           'PassportElementErrorDataField', 'PassportElementErrorFrontSide',
+           'PassportElementErrorReverseSide', 'PassportElementErrorSelfie',
+           'PassportElementErrorFile', 'PassportElementErrorFiles',
+           'PassportElementErrorTranslationFile',
            'PassportElementErrorTranslationFiles',
            'PassportElementErrorUnspecified', 'Game', 'CallbackGame',
            'GameHighScore')
@@ -262,6 +263,8 @@ class Update(BaseTelegram):
     pre_checkout_query: Optional['PreCheckoutQuery'] = None
     poll: Optional['Poll'] = None
     poll_answer: Optional['PollAnswer'] = None
+    my_chat_member: Optional['ChatMemberUpdated'] = None
+    chat_member: Optional['ChatMemberUpdated'] = None
 
 
 @attr.s(auto_attribs=True)
@@ -354,6 +357,7 @@ class Message(BaseTelegram):
     group_chat_created: Optional[bool] = None
     supergroup_chat_created: Optional[bool] = None
     channel_chat_created: Optional[bool] = None
+    message_auto_delete_timer_changed: Optional['MADTC'] = None
     migrate_to_chat_id: Optional[int] = None
     migrate_from_chat_id: Optional[int] = None
     pinned_message: Optional['Message'] = None
@@ -362,6 +366,10 @@ class Message(BaseTelegram):
     connected_website: Optional[str] = None
     passport_data: Optional['PassportData'] = None
     proximity_alert_triggered: Optional['ProximityAlertTriggered'] = None
+    voice_chat_scheduled: Optional['VoiceChatScheduled'] = None
+    voice_chat_started: Optional['VoiceChatStarted'] = None
+    voice_chat_ended: Optional['VoiceChatEnded'] = None
+    voice_chat_participants_invited: Optional['VCPI'] = None
     reply_markup: Optional['InlineKeyboardMarkup'] = None
 
 
@@ -490,6 +498,24 @@ class Venue(BaseTelegram):
     google_place_type: Optional[str] = None
 
 
+@attr.s(auto_attribs=True)
+class VoiceChatStarted(BaseTelegram):
+    pass
+
+
+@attr.s(auto_attribs=True)
+class VoiceChatEnded(BaseTelegram):
+    duration: int
+
+
+@attr.s(auto_attribs=True)
+class VoiceChatParticipantsInvited(BaseTelegram):
+    users: Optional[Tuple[User, ...]] = None
+
+
+VCPI = VoiceChatParticipantsInvited
+
+
 @attr.s(auto_attribs=True)
 class ProximityAlertTriggered(BaseTelegram):
     traveler: User
@@ -497,12 +523,26 @@ class ProximityAlertTriggered(BaseTelegram):
     distance: int
 
 
+@attr.s(auto_attribs=True)
+class MessageAutoDeleteTimerChanged(BaseTelegram):
+    message_auto_delete_time: int
+
+
+MADTC = MessageAutoDeleteTimerChanged
+
+
+@attr.s(auto_attribs=True)
+class VoiceChatScheduled(BaseTelegram):
+    start_date: int
+
+
 @attr.s(auto_attribs=True)
 class PollOption(BaseTelegram):
     text: str
     voter_count: int
 
 
+@attr.s(auto_attribs=True)
 class PollAnswer(BaseTelegram):
     poll_id: str
     user: User
@@ -517,7 +557,7 @@ class Poll(BaseTelegram):
     total_voter_count: int
     is_closed: bool
     is_anonymous: bool
-    type_: PollType
+    type_: str
     allows_multiple_answers: bool
     explanation_entities: Tuple[MessageEntity, ...]
     correct_option_id: Optional[int] = None
@@ -623,6 +663,16 @@ class ChatPhoto(BaseTelegram):
     big_file_unique_id: str
 
 
+@attr.s(auto_attribs=True)
+class ChatInviteLink(BaseTelegram):
+    invite_link: str
+    creator: User
+    is_primary: bool
+    is_revoked: bool
+    expire_date: Optional[int] = None
+    member_limit: Optional[int] = None
+
+
 @attr.s(auto_attribs=True)
 class ChatMember(BaseTelegram):
     user: User
@@ -631,10 +681,12 @@ class ChatMember(BaseTelegram):
     is_anonymous: Optional[bool] = None
     until_date: Optional[int] = None
     can_be_edited: Optional[bool] = None
+    can_manage_chat: Optional[bool] = None
     can_change_info: Optional[bool] = None
     can_post_messages: Optional[bool] = None
     can_edit_messages: Optional[bool] = None
     can_delete_messages: Optional[bool] = None
+    can_manage_voice_chats: Optional[bool] = None
     can_invite_users: Optional[bool] = None
     can_restrict_members: Optional[bool] = None
     can_pin_messages: Optional[bool] = None
@@ -647,6 +699,16 @@ class ChatMember(BaseTelegram):
     can_send_polls: Optional[bool] = None
 
 
+@attr.s(auto_attribs=True)
+class ChatMemberUpdated(BaseTelegram):
+    chat: Chat
+    from_: User
+    date: int
+    old_chat_member: ChatMember
+    new_chat_member: ChatMember
+    invite_link: Optional[ChatInviteLink] = None
+
+
 @attr.s(auto_attribs=True)
 class ChatPermissions(BaseTelegram):
     can_send_messages: Optional[bool] = None
@@ -766,6 +828,7 @@ class InlineQuery(BaseTelegram):
     from_: User
     query: str
     offset: str
+    chat_type: Optional[str] = None
     location: Optional[Location] = None
 
 
@@ -1127,6 +1190,30 @@ class InputContactMessageContent(BaseTelegram):
     vcard: Optional[str] = None
 
 
+@attr.s(auto_attribs=True)
+class InputInvoiceMessageContent:
+    title: str
+    description: str
+    payload: str
+    provider_token: str
+    currency: str
+    prices: Tuple['LabeledPrice', ...]
+    max_tip_amount: Optional[int] = None
+    suggested_tip_amounts: Optional[Tuple[int, ...]] = None
+    provider_data: Optional[str] = None
+    photo_url: Optional[str] = None
+    photo_size: Optional[int] = None
+    photo_width: Optional[int] = None
+    photo_height: Optional[int] = None
+    need_name: Optional[bool] = None
+    need_phone_number: Optional[bool] = None
+    need_email: Optional[bool] = None
+    need_shipping_address: Optional[bool] = None
+    send_phone_number_to_provider: Optional[bool] = None
+    send_email_to_provider: Optional[bool] = None
+    is_flexible: Optional[bool] = None
+
+
 @attr.s(auto_attribs=True)
 class ChosenInlineResult(BaseTelegram):
     result_id: str
@@ -1216,8 +1303,8 @@ class PassportData(BaseTelegram):
 class PassportFile(BaseTelegram):
     file_id: str
     file_unique_id: str
-    file_size: int
     file_date: int
+    file_size: Optional[int] = None
 
 
 @attr.s(auto_attribs=True)
